@@ -33,6 +33,8 @@ public class DCTDecode
   private static final int PUL = 2;
   private static final int PL = 1;
 
+  private short[] dequant_matrix = new short[64];
+
   private static final int[] ModeUsesMC = { 0, 0, 1, 1, 1, 0, 1, 1 };
 
   /* predictor multiplier up-left, up, up-right,left, shift
@@ -102,13 +104,30 @@ public class DCTDecode
     int     ReconPixelIndex;
     short[] dequant_coeffs;
 
+    /* determine which quantizer was specified for this block */
+    int qi = pbi.FragQs[FragmentNumber];
+    
     /* Select the appropriate inverse Q matrix and line stride */
     if ( FragmentNumber<(int)pbi.YPlaneFragments ){
       ReconPixelsPerLine = pbi.YStride;
-      dequant_coeffs = pbi.dequant_Y_coeffs;
-    }else{
+      // intra Y
+      dequant_coeffs = pbi.info.dequant_tables[0][0][pbi.frameQIS[qi]];
+      dequant_matrix[0] = pbi.info.dequant_tables[0][0][pbi.frameQIS[0]][0];
+    }else if(FragmentNumber < pbi.YPlaneFragments + pbi.UVPlaneFragments) {
       ReconPixelsPerLine = pbi.UVStride;
-      dequant_coeffs = pbi.dequant_UV_coeffs;
+      // intra U
+      dequant_coeffs = pbi.info.dequant_tables[0][1][pbi.frameQIS[qi]];
+      dequant_matrix[0] = pbi.info.dequant_tables[0][1][pbi.frameQIS[0]][0];
+    } else {
+      ReconPixelsPerLine = pbi.UVStride;
+      // intra V
+      dequant_coeffs = pbi.info.dequant_tables[0][2][pbi.frameQIS[qi]];
+      dequant_matrix[0] = pbi.info.dequant_tables[0][2][pbi.frameQIS[0]][0];
+    }
+
+    // create copy with DC coefficient of primary frame qi
+    for(int i = 1; i < 64; ++i) {
+        dequant_matrix[i] = dequant_coeffs[i];
     }
 
     /* Set up pointer into the quantisation buffer. */
@@ -117,13 +136,13 @@ public class DCTDecode
     /* Invert quantisation and DCT to get pixel data. */
     switch(pbi.FragCoefEOB[FragmentNumber]){
     case 0:case 1:
-      idct.IDct1(quantized_list, dequant_coeffs, ReconDataBuffer );
+      idct.IDct1(quantized_list, dequant_matrix, ReconDataBuffer );
       break;
     case 2: case 3:case 4:case 5:case 6:case 7:case 8: case 9:case 10:
-      idct.IDct10(quantized_list, dequant_coeffs, ReconDataBuffer );
+      idct.IDct10(quantized_list, dequant_matrix, ReconDataBuffer );
       break;
     default:
-      idct.IDctSlow(quantized_list, dequant_coeffs, ReconDataBuffer );
+      idct.IDctSlow(quantized_list, dequant_matrix, ReconDataBuffer );
     }
     /*
     for (int i=0; i<64; i++) {
@@ -154,6 +173,9 @@ public class DCTDecode
                                         pixel is used */
     short[] dequant_coeffs;
     CodingMode codingMode;
+    
+    /* determine which quantizer was specified for this block */
+    int qi = pbi.FragQs[FragmentNumber];
 
     /* Get coding mode for this block */
     if (pbi.getFrameType() == Constants.BASE_FRAME ){
@@ -170,35 +192,63 @@ public class DCTDecode
       MvModMask = 0x00000001;
 
       /* Select appropriate dequantiser matrix. */
-      if ( codingMode == CodingMode.CODE_INTRA )
-        dequant_coeffs = pbi.dequant_Y_coeffs;
-      else
-        dequant_coeffs = pbi.dequant_Inter_coeffs;
+      if ( codingMode == CodingMode.CODE_INTRA ) {
+        // intra Y
+        dequant_coeffs = pbi.info.dequant_tables[0][0][pbi.frameQIS[qi]];
+        dequant_matrix[0] = pbi.info.dequant_tables[0][0][pbi.frameQIS[0]][0];
+      } else {
+        // inter Y
+        dequant_coeffs = pbi.info.dequant_tables[1][0][pbi.frameQIS[qi]];
+        dequant_matrix[0] = pbi.info.dequant_tables[1][0][pbi.frameQIS[0]][0];
+      }
     }else{
       ReconPixelsPerLine = pbi.UVStride;
       MvShift = 2;
       MvModMask = 0x00000003;
 
       /* Select appropriate dequantiser matrix. */
-      if ( codingMode == CodingMode.CODE_INTRA )
-        dequant_coeffs = pbi.dequant_UV_coeffs;
-      else
-        dequant_coeffs = pbi.dequant_Inter_coeffs;
-    }
 
+
+      if(FragmentNumber < pbi.YPlaneFragments + pbi.UVPlaneFragments) {
+        if ( codingMode == CodingMode.CODE_INTRA ) {
+          // intra U
+          dequant_coeffs = pbi.info.dequant_tables[0][1][pbi.frameQIS[qi]];
+          dequant_matrix[0] = pbi.info.dequant_tables[0][1][pbi.frameQIS[0]][0];
+        } else {
+          // inter U
+          dequant_coeffs = pbi.info.dequant_tables[1][1][pbi.frameQIS[qi]];
+          dequant_matrix[0] = pbi.info.dequant_tables[1][1][pbi.frameQIS[0]][0];
+        }
+      } else {
+        if ( codingMode == CodingMode.CODE_INTRA ) {
+          // intra V
+          dequant_coeffs = pbi.info.dequant_tables[0][2][pbi.frameQIS[qi]];
+          dequant_matrix[0] = pbi.info.dequant_tables[0][2][pbi.frameQIS[0]][0];
+        } else {
+          // inter V
+          dequant_coeffs = pbi.info.dequant_tables[1][2][pbi.frameQIS[qi]];
+          dequant_matrix[0] = pbi.info.dequant_tables[1][2][pbi.frameQIS[0]][0];
+        }
+      }
+    }
+    // create copy with DC coefficient of primary frame qi
+    for(int i = 1; i < 64; ++i) {
+        dequant_matrix[i] = dequant_coeffs[i];
+    }
+    
     /* Set up pointer into the quantisation buffer. */
     short[] quantized_list = pbi.QFragData[FragmentNumber];
 
     /* Invert quantisation and DCT to get pixel data. */
     switch(pbi.FragCoefEOB[FragmentNumber]){
     case 0:case 1:
-      idct.IDct1(quantized_list, dequant_coeffs, ReconDataBuffer );
+      idct.IDct1(quantized_list, dequant_matrix, ReconDataBuffer );
       break;
     case 2: case 3:case 4:case 5:case 6:case 7:case 8: case 9:case 10:
-      idct.IDct10(quantized_list, dequant_coeffs, ReconDataBuffer );
+      idct.IDct10(quantized_list, dequant_matrix, ReconDataBuffer );
       break;
     default:
-      idct.IDctSlow(quantized_list, dequant_coeffs, ReconDataBuffer );
+      idct.IDctSlow(quantized_list, dequant_matrix, ReconDataBuffer );
     }
     /*
     for (int i=0; i<64; i++) {
@@ -616,17 +666,6 @@ public class DCTDecode
     }
   }
 
-  public void ClearDownQFragData(Playback pbi){
-    int       i,j;
-    short[]    QFragPtr;
-
-    for ( i = 0; i < pbi.CodedBlockIndex; i++ ) {
-      /* Get the linear index for the current fragment. */
-      QFragPtr = pbi.QFragData[pbi.CodedBlockList[i]];
-      for ( j = 0; j < 64; j++ ) QFragPtr[j]  = 0;
-    }
-  }
-
   public void ReconRefFrames (Playback pbi){
     int i;
     int j,k,m,n;
